/*****************************************************************************
 *
 * CHECKS.C - Service and host check functions for NetSaint
 *
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified:   01-31-2000
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netsaint.h"

extern char     temp_file[MAX_FILENAME_LENGTH];

extern int      interval_length;
extern int      max_child_process_time;
extern int      ipc_message_queue;
extern int      child_process_timed_out;
extern int      use_agressive_host_checking;

extern int      currently_running_service_checks;

extern timed_event     *event_list_low;

extern host     *host_list;

extern service_message svc_msg;


/******************************************************************/
/****************** SERVICE MONITORING FUNCTIONS ******************/
/******************************************************************/

/* forks a child process to run a service check, but does not wait for the service check result */
void run_service_check(service *svc){
	char raw_command[MAX_COMMANDLINE_LENGTH];
	char processed_command[MAX_COMMANDLINE_LENGTH];
	char plugin_output[MAX_PLUGINOUTPUT_LENGTH];
	int check_service=TRUE;
	time_t current_time;
	time_t preferred_time=0L;
	time_t next_valid_time;
	pid_t pid;
	int fork_error=FALSE;
	int wait_result=0;
	int result=0;
	host *temp_host=NULL;
	FILE *fp;
	int pclose_result=0;
	timed_event *new_event=NULL;


	/* get the current time */
	time(&current_time);

	/* if the service check is currently disabled... */
	if(svc->disabled==TRUE){

		/* don't check the service */
		check_service=FALSE;

		/* reschedule the service check */
		preferred_time=current_time+(svc->check_interval*interval_length);
                }

	/* make sure this is a valid time to check the service */
	if(!check_time_against_period((unsigned long)current_time,svc->check_period)){

		/* don't check the service */
		check_service=FALSE;

		/* get the next valid time we can run the check */
		preferred_time=current_time;
	        }

	/* find the host associated with this service */
	temp_host=find_host(svc->host_name,NULL);

	/* don't check the service if we couldn't find the associated host */
	if(temp_host==NULL)
		check_service=FALSE;

	/* if we shouldn't check the service, just reschedule it and leave... */
	if(check_service==FALSE){

		/* make sure we rescheduled the next service check at a valid time */
		get_next_valid_time(preferred_time,&next_valid_time,svc->check_period);
		svc->next_check=next_valid_time;

		/* update the status log with the current service */
		update_status_log_with_service(svc);

		/* reschedule the next service check */
		new_event=malloc(sizeof(timed_event));
		if(new_event!=NULL){
			new_event->event_type=EVENT_SERVICE_CHECK;
			new_event->event_data=(void *)svc;
			new_event->run_time=svc->next_check;
			new_event->recurring=FALSE;
			schedule_event(new_event,&event_list_low);
		        }

		return;
	        }


	/**** ELSE RUN THE SERVICE CHECK ****/

#ifdef DEBUG3
	printf("\tChecking service '%s' on host '%s'...\n",svc->description,svc->host_name);
#endif

	/* increment number of parallel service checks currently out there... */
	currently_running_service_checks++;

	/* set the last service check time */
	svc->last_check=current_time;

	/* clear all the volatile macros */
	clear_volatile_macros();

	/* grab the host and service macro variables */
	grab_host_macros(temp_host);
	grab_service_macros(svc);

	/* get the raw command line */
	get_raw_command_line(svc->command,raw_command,sizeof(raw_command));
	strip(raw_command);

	/* process any macros contained in the argument */
	process_macros(raw_command,processed_command,sizeof(processed_command));
	strip(processed_command);

	/* save service info */
	strncpy(svc_msg.host_name,svc->host_name,sizeof(svc_msg.host_name)-1);
	svc_msg.host_name[sizeof(svc_msg.host_name)-1]='\x0';
	strncpy(svc_msg.description,svc->description,sizeof(svc_msg.description)-1);
	svc_msg.description[sizeof(svc_msg.description)-1]='\x0';

	/* fork a child process */
	pid=fork();

	/* an error occurred while trying to fork */
	if(pid==-1)
		fork_error=TRUE;

	/* if we are in the child process... */
	else if(pid==0){

		/* become the process group leader */
		setpgid(0,0);

		/* free allocated memory */
		free_memory();

		/* fork again... */
		pid=fork();

		/* an error occurred while trying to fork again */
		if(pid==-1)
			exit(STATE_UNKNOWN);

		/* the grandchild process should run the service check... */
		if(pid==0){

			/* reset signal handling */
			reset_sighandler();

			/* catch plugins that don't finish in a timely manner */
			signal(SIGALRM,service_check_sighandler);
			alarm(DEFAULT_MAX_CHILD_PROCESS_TIME);

			/* run the plugin check command */
			fp=popen(processed_command,"r");
			if(fp==NULL)
				exit(STATE_UNKNOWN);

			/* grab the plugin output and clean it */
			fgets(plugin_output,sizeof(plugin_output)-1,fp);
			strip(plugin_output);

			/* close the process */
			pclose_result=pclose(fp);

			/* reset the alarm */
			alarm(0);

			/* test for execution error */
			if(pclose_result==-1){
				strncpy(svc_msg.output,"(Error returned by call to pclose() function)",sizeof(svc_msg.output)-1);
				svc_msg.output[sizeof(svc_msg.output)-1]='\x0';
				svc_msg.return_code=STATE_CRITICAL;
				svc_msg.exited_ok=FALSE;
				send_svc_message(ipc_message_queue,&svc_msg);

				exit(STATE_UNKNOWN);
			        }

			/* else write plugin check results to message queue */
			else{
				strncpy(svc_msg.output,plugin_output,sizeof(svc_msg.output)-1);
				svc_msg.output[sizeof(svc_msg.output)-1]='\x0';
				svc_msg.return_code=(int)WEXITSTATUS(pclose_result);
				svc_msg.exited_ok=TRUE;
				send_svc_message(ipc_message_queue,&svc_msg);
			        }

			/* return with plugin exit status - not really necessary... */
			exit(pclose_result);
		        }

		/* parent exits immediately - grandchild process is inherited by the INIT process, so we have no zombie problem... */
		exit(STATE_OK);
	        }

	/* else the parent should wait for the first child to return... */
	else if(pid>0){

		wait_result=waitpid(pid,NULL,0);

		result=WEXITSTATUS(wait_result);
		if(result==STATE_UNKNOWN)
			fork_error=TRUE;
	        }

	/* see if we could run the check... */
	if(fork_error==TRUE){

		/* write plugin check results to message queue */
		strncpy(svc_msg.output,"(Could not execute service check due to a fork() error)",sizeof(svc_msg.output)-1);
		svc_msg.output[sizeof(svc_msg.output)-1]='\x0';
		svc_msg.return_code=STATE_CRITICAL;
		svc_msg.exited_ok=TRUE;
		send_svc_message(ipc_message_queue,&svc_msg);
	        }
	
	return;
        }



/* reaps service check results */
void reap_service_checks(void){
	service_message svc_msg;
	service *temp_service=NULL;
	host *temp_host=NULL;
	timed_event *new_event=NULL;
	unsigned long preferred_time=0L;
	unsigned long next_valid_time=0L;
	char temp_buffer[MAX_INPUT_BUFFER];
	int state_change=FALSE;
	int hard_state_change=FALSE;
	int route_result=HOST_UP;
	int ignore_service_for_now=FALSE;
	time_t current_time;

#ifdef DEBUG0
        printf("reap_service_checks() start\n");
#endif

#ifdef DEBUG3
	printf("Starting to reap service check results...\n");
#endif
	
	while(read_svc_message(ipc_message_queue,&svc_msg)!=-1){

		/* because of my idiotic idea of having UNKNOWN states be equivalent to -1, I must hack things a bit... */
		if(svc_msg.return_code==255)
			svc_msg.return_code=STATE_UNKNOWN;

		/* find the service */
		temp_service=find_service(svc_msg.host_name,svc_msg.description,NULL);
		if(temp_service==NULL)
			continue;

#ifdef DEBUG3
		printf("\n\tFound check result for service '%s' on host '%s'\n",temp_service->host_name,temp_service->description);
		printf("\t\tExited OK?:    %s\n",(svc_msg.exited_ok==TRUE)?"Yes":"No");
		printf("\t\tReturn Status: %d\n",svc_msg.return_code);
		printf("\t\tPlugin Output: '%s'\n",svc_msg.output);
#endif

		/* decrement the number of service checks still out there... */
		currently_running_service_checks--;

		/* INITIALIZE VARIABLES FOR THIS SERVICE */
		state_change=FALSE;
		hard_state_change=FALSE;
		route_result=HOST_UP;
		ignore_service_for_now=FALSE;

		/* save the old service status info */
		temp_service->last_state=temp_service->current_state;

		/* if there was some error running the command, just skip it (this shouldn't be happening) */
		if(svc_msg.exited_ok==FALSE){

			snprintf(temp_buffer,sizeof(temp_buffer),"Warning:  Check of service '%s' on host '%s' did not exit properly!\n",temp_service->description,temp_service->host_name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
#ifdef DEBUG3
			printf("%s",temp_buffer);
#endif
			write_to_log(temp_buffer);

			snprintf(temp_service->plugin_output,MAX_PLUGINOUTPUT_LENGTH-1,"(Service check did not exit properly)");
			temp_service->plugin_output[MAX_PLUGINOUTPUT_LENGTH-1]='\x0';

			temp_service->current_state=STATE_CRITICAL;
                        }

		/* make sure the return code is within bounds */
		else if(svc_msg.return_code<-1 || svc_msg.return_code>2){

			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Return code of %d for check of service '%s' on host '%s' was out of bounds.\n",svc_msg.return_code,temp_service->description,temp_service->host_name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
#ifdef DEBUG3
			printf("%s",temp_buffer);
#endif
			write_to_log(temp_buffer);

			snprintf(temp_service->plugin_output,MAX_PLUGINOUTPUT_LENGTH-1,"(Return code of %d is out of bounds)",svc_msg.return_code);
			temp_service->plugin_output[MAX_PLUGINOUTPUT_LENGTH-1]='\x0';

			temp_service->current_state=STATE_CRITICAL;
                        }

		/* else the return code is okay... */
		else{

			/* grab the plugin output */
			strncpy(temp_service->plugin_output,svc_msg.output,MAX_PLUGINOUTPUT_LENGTH-1);
			temp_service->plugin_output[MAX_PLUGINOUTPUT_LENGTH-1]='\x0';

			/* grab the return code */
			temp_service->current_state=svc_msg.return_code;
		        }

		/* get the host that this service runs on */
		temp_host=find_host(temp_service->host_name,NULL);

		/* if the host has never been checked before, update the last check time */
		if((unsigned long)temp_host->last_check==0L && temp_service->current_state==STATE_OK){

			temp_host->last_check=temp_service->last_check;
			temp_host->last_state_change=temp_service->last_check;

			strncpy(temp_host->plugin_output,"(Host assumed to be up)",MAX_PLUGINOUTPUT_LENGTH);
			temp_host->plugin_output[MAX_PLUGINOUTPUT_LENGTH-1]='\x0';

			/* update the status log with the host status */
			update_status_log_with_host(temp_host);
		        }

		/* if the host has never been checked before, update the last state change time */
		if((unsigned long)temp_host->last_state_change==0L && temp_service->current_state==STATE_OK){

			temp_host->last_state_change=temp_service->last_check;

			/* update the status log with the host status */
			update_status_log_with_host(temp_host);
		        }

		/* if the service has never been checked before, update the last state change time */
		if((unsigned long)temp_service->last_state_change==0L && temp_service->current_state==STATE_OK)
			temp_service->last_state_change=temp_service->last_check;

		/* check for a state change (either soft or hard) */
		if(temp_service->current_state!=temp_service->last_state){
		
#ifdef DEBUG3
			printf("\t\tService '%s' on host '%s' has changed state since last check!\n",temp_service->description,temp_service->host_name);
#endif

			/* set the state change flag */
			state_change=TRUE;
		        }


		/* checks for a hard state change where host was down at last service check */
		if(temp_service->host_problem_at_last_check==TRUE && temp_service->current_state==STATE_OK){

#ifdef DEBUG3
			printf("\t\tService '%s' on host '%s' has had a HARD STATE CHANGE!!\n",temp_service->description,temp_service->host_name);
#endif

			hard_state_change=TRUE;
	                }


		/* check for a "normal" hard state change where max check attempts is reached */
		else if(temp_service->current_attempt>=temp_service->max_attempts){

			if(temp_service->current_state!=temp_service->last_hard_state){

#ifdef DEBUG3
				printf("\t\tService '%s' on host '%s' has had a HARD STATE CHANGE!!\n",temp_service->description,temp_service->host_name);
#endif

				hard_state_change=TRUE;
			        }
		        }

		/* reset last notification time if necessary */
		if(state_change==TRUE || hard_state_change==TRUE)
			temp_service->last_notification=0L;






		/**************************************/
		/******* SERVICE CHECK OK LOGIC *******/
		/**************************************/

		/* if the service is up and running OK... */
		if(temp_service->current_state==STATE_OK){

			/* the service check was okay, so the associated host must be up... */
			if(temp_host->status!=HOST_UP){

				/* verify the route to the host and send out host recovery notifications */
				verify_route_to_host(temp_host);

				/* set the host problem flag (i.e. don't notify about recoveries for this service) */
				temp_service->host_problem_at_last_check=TRUE;
			        }

			/* if a hard service recovery has occurred... */
			if(hard_state_change==TRUE){

				/* set the state type macro */
				temp_service->state_type=HARD_STATE;

				/* update service state times... */
				update_service_state_times(temp_service);

				/* log the service recovery */
				log_service_event(temp_service,HARD_STATE);

				/* notify contacts about the service recovery if the host was not down or unreachable at the last check */
				if(temp_service->notify_on_recovery  && temp_service->host_problem_at_last_check==FALSE){

					if(temp_service->last_state==STATE_UNKNOWN && temp_service->notify_on_warning)
						service_notification(temp_service);
		
					else if(temp_service->last_state==STATE_WARNING && temp_service->notify_on_warning)
						service_notification(temp_service);

					else if(temp_service->last_state==STATE_CRITICAL && temp_service->notify_on_critical)
						service_notification(temp_service);
				        }

				/* run the service event handler to handle the hard state change */
				handle_service_event(temp_service,HARD_STATE);
			        }

			/* else if a soft service recovery has occurred... */
			else if(state_change==TRUE){

				/* this is a soft recovery */
				temp_service->state_type=SOFT_STATE;

				/* log the soft recovery */
				log_service_event(temp_service,SOFT_STATE);

				/* run the service event handler to handle the soft state change */
				handle_service_event(temp_service,SOFT_STATE);
		                }

			/* else no service state change has occured... */

			/* reset all service variables because its okay now... */
			temp_service->last_notification=0L;
			temp_service->host_problem_at_last_check=FALSE;
			temp_service->state_type=HARD_STATE;
			temp_service->current_attempt=1;
			temp_service->next_check=temp_service->last_check+(temp_service->check_interval*interval_length);
			temp_service->last_hard_state=STATE_OK;
		        }


		/*******************************************/
		/******* SERVICE CHECK PROBLEM LOGIC *******/
		/*******************************************/

		/* hey, something's not working quite like it should... */
		else{

			/* check the route to the host if its supposed to be up right now... */
			if(temp_host->status==HOST_UP)
				route_result=verify_route_to_host(temp_host);

			/* else the host is either down or unreachable, so recheck it if necessary */
			else{

				/* we're using agressive host checking, so really do recheck the host... */
				if(use_agressive_host_checking)
					route_result=verify_route_to_host(temp_host);

				/* else fake the host check, but (possibly) resend host notifications to contacts... */
				else{
					route_result=temp_host->status;
					host_notification(temp_host,temp_host->status);
				        }
			        }

			/* if the host is down or unreachable ... */
			if(route_result!=HOST_UP){

				/* "fake" a hard state change for the service - well, its not really fake, but it didn't get caught earler... */
				if(temp_service->last_hard_state!=temp_service->current_state)
					hard_state_change=TRUE;

				/* update service state times if necessary */
				if(hard_state_change==TRUE)
					update_service_state_times(temp_service);

				/* put service into a hard state without attempting check retries and don't send out notifications about it */
				temp_service->host_problem_at_last_check=TRUE;
				temp_service->state_type=HARD_STATE;
				temp_service->last_hard_state=temp_service->current_state;
				temp_service->current_attempt=1;
				ignore_service_for_now=TRUE;

				/* if the service has never been checked before, update the last state change time */
				if((unsigned long)temp_service->last_state_change==0L)
					temp_service->last_state_change=temp_service->last_check;
			        }

			/* else the host is up.. */
			else{

				/*  the host recovered since the last time the service was checked... */
				if(temp_service->host_problem_at_last_check==TRUE){

					/* next time the service is checked we shouldn't get into this same case... */
					temp_service->host_problem_at_last_check=FALSE;

					/* reset the current check counter, so we can mabye avoid a false recovery alarm - added 07/28/99 */
					temp_service->current_attempt=1;

					/* don't send out notifications for this service, since the host just came back online... */
					ignore_service_for_now=TRUE;
				        }
			        }

			 /* if we should retry the service check, do so (except it the host is down or unreachable!) */
			if(temp_service->current_attempt < temp_service->max_attempts){

				/* the host is down or unreachable, so don't attempt to retry the service check */
				if(route_result!=HOST_UP){

					/* the host is not up, so reschedule the next service check at regular interval */
					temp_service->next_check=temp_service->last_check+(temp_service->check_interval*interval_length);

					/* log the problem as a hard state if the host just went down (new in 0.0.5) */
					if(hard_state_change==TRUE)
						log_service_event(temp_service,HARD_STATE);
				        }

				/* the host is up, so continue to retry the service check */
				else{

					/* this is a soft state */
					temp_service->state_type=SOFT_STATE;

					/* log the service check retry */
					log_service_event(temp_service,SOFT_STATE);

					/* run the service event handler to handle the soft state */
					handle_service_event(temp_service,SOFT_STATE);

					temp_service->current_attempt=temp_service->current_attempt+1;
					temp_service->next_check=temp_service->last_check+(temp_service->retry_interval*interval_length);
				        }
			        }
			

			/* we've reached the maximum number of service rechecks, so handle the error */
			else{

				/* this is a hard state */
				temp_service->state_type=HARD_STATE;

				/* if we've hard a hard state change... */
				if(hard_state_change==TRUE){

					/* update service state times */
					update_service_state_times(temp_service);

					/* log the service problem (even if host is not up, which is new in 0.0.5) */
					log_service_event(temp_service,HARD_STATE);
				        }

				/* (re)send notifications out about this service problem if the host is up... */
				if(ignore_service_for_now==FALSE){

					if(temp_service->current_state==STATE_WARNING && temp_service->notify_on_warning==TRUE)
						service_notification(temp_service);

					else if(temp_service->current_state==STATE_CRITICAL && temp_service->notify_on_critical==TRUE)
						service_notification(temp_service);

					else if(temp_service->current_state==STATE_UNKNOWN && temp_service->notify_on_warning==TRUE)
						service_notification(temp_service);
					        }

				/* run the service event handler if we changed state from the last hard state */
				if(hard_state_change==TRUE)
					handle_service_event(temp_service,HARD_STATE);

				/* save the last hard state */
				temp_service->last_hard_state=temp_service->current_state;

				/* reschedule the next check at the regular interval */
				temp_service->next_check=temp_service->last_check+(temp_service->check_interval*interval_length);       
			        }
		        }

	        /* make sure we don't get ourselves into too much trouble... */
		time(&current_time);
		if(current_time>temp_service->next_check)
			temp_service->next_check=current_time;

		/* make sure we rescheduled the next service check at a valid time */
		preferred_time=(unsigned long)temp_service->next_check;
		get_next_valid_time(preferred_time,&next_valid_time,temp_service->check_period);
		temp_service->next_check=(time_t)next_valid_time;

		/* update the current service status log */
		update_status_log_with_service(temp_service);

		/* reschedule the next service check */
		new_event=malloc(sizeof(timed_event));
		if(new_event!=NULL){
			new_event->event_type=EVENT_SERVICE_CHECK;
			new_event->event_data=(void *)temp_service;
			new_event->run_time=temp_service->next_check;
			new_event->recurring=FALSE;
			schedule_event(new_event,&event_list_low);
	                }
	        }

#ifdef DEBUG3
	printf("Finished reaping service check results.\n");
#endif

#ifdef DEBUG0
	printf("reap_service_checks() end\n");
#endif

	return;
        }




/******************************************************************/
/******************* ROUTE/HOST CHECK FUNCTIONS *******************/
/******************************************************************/


/* check to see if we can reach the host */
int verify_route_to_host(host *hst){
	int result;

#ifdef DEBUG0
	printf("verify_route_to_host() start\n");
#endif

	/* check route to the host (propagate problems and recoveries both up and down the tree) */
	result=check_host(hst,PROPAGATE_TO_PARENT_HOSTS | PROPAGATE_TO_CHILD_HOSTS);

#ifdef DEBUG0
	printf("verify_route_to_host() end\n");
#endif

	return result;
        }



/* see if the remote host is alive at all */
int check_host(host *hst,int propagation_options){
	int result=HOST_UP;
	int parent_result=HOST_UP;
	host *parent_host=NULL;
	host *child_host=NULL;
	int return_result=HOST_UP;
	int max_check_attempts=1;

#ifdef DEBUG0
	printf("check_host() start\n");
#endif

	/* make sure we return the original host state unless it changes... */
	return_result=hst->status;

	/* if the host is already down or unreachable... */
	if(hst->status!=HOST_UP){

		/* how many times should we retry checks for this host? */
		if(!use_agressive_host_checking)
			max_check_attempts=1;
		else
			max_check_attempts=hst->max_attempts;

		/* retry the host check as many times as necessary or allowed... */
		for(hst->current_attempt=1;hst->current_attempt<=max_check_attempts;hst->current_attempt++){
			
			/* check the host */
			result=run_host_check(hst);

			/* the host recovered from a hard problem... */
			if(result==HOST_UP){

				return_result=HOST_UP;

				/* handle the hard host recovery */
				handle_host_state(hst,HOST_UP,HARD_STATE);

				/* propagate the host recovery upwards (parents should not be down) */
				if(propagation_options & PROPAGATE_TO_PARENT_HOSTS){

					/* find the parent host */
					parent_host=find_host(hst->parent_host,NULL);

					/* check the parent host (and propagate upwards) if its not up */
					if(parent_host!=NULL && parent_host->status!=HOST_UP)
						check_host(parent_host,PROPAGATE_TO_PARENT_HOSTS);
				        }

				/* propagate the host recovery downwards (children may or may not be down) */
				if(propagation_options & PROPAGATE_TO_CHILD_HOSTS){

					/* check all child hosts... */
					for(child_host=host_list;child_host!=NULL;child_host=child_host->next){

						/* if this is a child of the host, check it if it is not marked as UP */
						if(is_immediate_child_of_host(hst,child_host)==TRUE && child_host->status!=HOST_UP)
						        check_host(child_host,PROPAGATE_TO_CHILD_HOSTS);
					        }
				        }

				break;
			        }

		        }

		/* reset the current attempt if we just went over... */
		if(hst->current_attempt>hst->max_attempts)
			hst->current_attempt=hst->max_attempts;
	        }


	/* else the host is supposed to be up right now... */
	else{

		for(hst->current_attempt=1;hst->current_attempt<=hst->max_attempts;hst->current_attempt++){

			/* run the host check */
			result=run_host_check(hst);

			/* if this is the last check and we still haven't had a recovery, check the parents and children and then handle the hard host state */
			if(result!=HOST_UP && (hst->current_attempt==hst->max_attempts)){

			        /* find the parent host */
				parent_host=find_host(hst->parent_host,NULL);

			        /* check the parent host, assume its up if we can't find it, use the parent host's "old" status if we shouldn't propagate */
				if(parent_host==NULL)
					parent_result=HOST_UP;
				else if(propagation_options & PROPAGATE_TO_PARENT_HOSTS)
					parent_result=check_host(parent_host,PROPAGATE_TO_PARENT_HOSTS);
				else
					parent_result=parent_host->status;

			        /* if the parent is down or unreachable, this host is unreachable */
				if(parent_result!=HOST_UP)
					return_result=HOST_UNREACHABLE;

			        /* else the parent host is up, so this host must be down */
				else
					return_result=HOST_DOWN;

				/* handle the hard host state (whether it is DOWN or UNREACHABLE) */
				handle_host_state(hst,return_result,HARD_STATE);

				/* propagate the host problem to all child hosts (they should be unreachable now unless they have multiple parents) */
				if(propagation_options & PROPAGATE_TO_CHILD_HOSTS){

					/* check all child hosts... */
					for(child_host=host_list;child_host!=NULL;child_host=child_host->next){

						/* if this is a child of the host, check it if it is not marked as UP */
						if(is_immediate_child_of_host(hst,child_host)==TRUE && child_host->status!=HOST_UP)
						        check_host(child_host,PROPAGATE_TO_CHILD_HOSTS);
					        }
				        }
			        }

			/* handle any soft states (during host check retries that return a non-ok state) */
			else if(result!=HOST_UP || (result==HOST_UP && hst->current_attempt!=1))
				handle_host_state(hst,result,SOFT_STATE);

			/* the host recovered (or it was never down), so break out of the check loop */
			if(result==HOST_UP){

				/* this host hasn't had a state change yet... */
				if((unsigned long)hst->last_state_change==0L){

					/* initialize the last state change time */
					hst->last_state_change=hst->last_check;

					/* update the status log with the current host info */
					update_status_log_with_host(hst);
				        }
				break;
			        }
	                }


		/* adjust the current check number if we exceeded the max count */
		if(hst->current_attempt>hst->max_attempts)
			hst->current_attempt=hst->max_attempts;
	        }


#ifdef DEBUG3
	printf("\tHost Check Result: Host '%s' is ",hst->name);
	if(return_result==HOST_UP)
		printf("UP\n");
	else if(return_result==HOST_DOWN)
		printf("DOWN\n");
	else if(return_result==HOST_UNREACHABLE)
		printf("UNREACHABLE\n");
#endif

#ifdef DEBUG0
	printf("check_host() end\n");
#endif
	
	return return_result;
        }




/* run an "alive" check on a host */
int run_host_check(host *hst){
	int result=STATE_OK;
	int return_result=HOST_UP;
	char processed_check_command[MAX_COMMANDLINE_LENGTH];
	char temp_buffer[MAX_COMMANDLINE_LENGTH];
	char raw_check_command[MAX_COMMANDLINE_LENGTH];
	command *temp_command;
	time_t current_time;
	char *temp_ptr;
	FILE *fp;

#ifdef DEBUG0
	printf("run_host_check() start\n");
#endif

	/* if there is no host check command, just return with no error */
	if(!strcmp(hst->host_check_command,"")){

#ifdef DEBUG3
		printf("\tNo host check command specified, so no check will be done!\n");
#endif

		return HOST_UP;
	        }

	/* find the command we use to check the host */
	temp_command=find_command(hst->host_check_command,NULL);

	/* if we couldn't find the command, return with an error */
	if(temp_command==NULL){

#ifdef DEBUG3
		printf("\tCouldn't find the host check command!\n");
#endif

		return HOST_UP;
	        }

	/* grab the host macros */
	grab_host_macros(hst);

	/* get the last host check time */
	time(&current_time);
	hst->last_check=current_time;

	/* get the raw command line */
	strncpy(raw_check_command,temp_command->command_line,sizeof(raw_check_command));
	raw_check_command[sizeof(raw_check_command)-1]='\x0';

	/* process any macros in the check command */
	process_macros(raw_check_command,&processed_check_command[0],(int)sizeof(processed_check_command));

	/* redirect output and errors */
	strncpy(temp_buffer,processed_check_command,sizeof(temp_buffer));
	temp_buffer[sizeof(temp_buffer)-1]='\x0';
	snprintf(processed_check_command,sizeof(processed_check_command),"%s > %s 2> /dev/null",temp_buffer,temp_file);
	processed_check_command[sizeof(processed_check_command)-1]='\x0';

			
#ifdef DEBUG3
	printf("\t\tRaw Command: %s\n",hst->host_check_command);
	printf("\t\tProcessed Command: %s\n",processed_check_command);
#endif

	/* reset the child process timeout flag */
	child_process_timed_out=FALSE;

	/* run the host check command */
	result=my_system(processed_check_command);

	/* if the child process timed out, report an error */
	if(child_process_timed_out==TRUE){
		snprintf(hst->plugin_output,MAX_PLUGINOUTPUT_LENGTH-1,"Host check timed out after %d seconds\n",max_child_process_time);
		hst->plugin_output[MAX_PLUGINOUTPUT_LENGTH-1]='\x0';
	        }

	/* else grab the first line of the plugin output */
	else{
		strcpy(hst->plugin_output,"");

		fp=fopen(temp_file,"r");
		if(fp!=NULL){
			fgets(hst->plugin_output,MAX_PLUGINOUTPUT_LENGTH-1,fp);
			hst->plugin_output[MAX_PLUGINOUTPUT_LENGTH-1]='\x0';
			fclose(fp);
		        }
	        }

	/* make sure the output isn't NULL */
	strip(hst->plugin_output);
	if(!strcmp(hst->plugin_output,"")){
		strncpy(hst->plugin_output,"(No Information Returned From Plugin)",MAX_PLUGINOUTPUT_LENGTH-1);
		hst->plugin_output[MAX_PLUGINOUTPUT_LENGTH-1]='\x0';
	        }

	/* replace semicolons in plugin output with colons */
	temp_ptr=hst->plugin_output;
	while((temp_ptr=strchr(temp_ptr,';')))
	      *temp_ptr=':';

	/* remove the temp file */
	remove(temp_file);

	/* if we're not doing agressive host checking, let WARNING states indicate the host is up (fake the result to be STATE_OK) */
	if(!use_agressive_host_checking && result==STATE_WARNING)
		result=STATE_OK;


	if(result==STATE_OK)
		return_result=HOST_UP;
	else
		return_result=HOST_DOWN;


#ifdef DEBUG3
	printf("\tHost Check Result: Host '%s' is ",hst->name);
	if(return_result==HOST_UP)
		printf("UP\n");
	else
		printf("DOWN\n");
#endif

#ifdef DEBUG0
	printf("run_host_check() end\n");
#endif
	
	return return_result;
        }

