/*****************************************************************************
 *
 * SEHANDLERS.C - Service and host event and state handlers for NetSaint
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified:   01-14-2000
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netsaint.h"


extern int             log_event_handlers;
extern int             log_host_retries;

extern char            macro_current_service_attempt[MAX_ATTEMPT_LENGTH];
extern char            macro_current_host_attempt[MAX_ATTEMPT_LENGTH];
extern char            macro_host_state[MAX_STATE_LENGTH];
extern char	       macro_service_state[MAX_STATE_LENGTH];
extern char            macro_state_type[MAX_STATETYPE_LENGTH];

extern char            global_host_event_handler[MAX_COMMANDNAME_LENGTH];
extern char            global_service_event_handler[MAX_COMMANDNAME_LENGTH];


/******************************************************************/
/**************** SERVICE EVENT HANDLER FUNCTIONS *****************/
/******************************************************************/


/* handles changes in the state of a service */
int handle_service_event(service *svc,int state_type){
	command *event_command;

#ifdef DEBUG0
	printf("handle_service_event() start\n");
#endif

	/* update service macros */
	grab_service_macros(svc);

	/* grab the service state type macro */
	strncpy(macro_state_type,(state_type==HARD_STATE)?"HARD":"SOFT",sizeof(macro_state_type)-1);
	macro_state_type[sizeof(macro_state_type)-1]='\x0';

	/* grab the current service check number macro */
	snprintf(macro_current_service_attempt,sizeof(macro_current_service_attempt),"%d",svc->current_attempt);
	macro_current_service_attempt[sizeof(macro_current_service_attempt)-1]='\x0';

	/* run the global service event handler */
	run_global_service_event_handler(svc,state_type);

	/* find the service event handler command if there is one */
	if(!strcmp(svc->event_handler,""))
		event_command=NULL;
	else
		event_command=find_command(svc->event_handler,NULL);

	/* run the event handler command if there is one */
	if(event_command!=NULL)
		run_service_event_handler(svc,event_command,state_type);

	/* check for external commands - the event handler may have given us some directives... */
	check_for_external_commands();

#ifdef DEBUG0
	printf("handle_service_event() end\n");
#endif

	return OK;
        }



/* runs the global service event handler */
int run_global_service_event_handler(service *svc,int state_type){
	char raw_command_line[MAX_COMMANDLINE_LENGTH];
	char processed_command_line[MAX_COMMANDLINE_LENGTH];
	char temp_buffer[MAX_INPUT_BUFFER];
	command *temp_command;

#ifdef DEBUG0
	printf("run_global_service_event_handler() start\n");
#endif

	/* find the event handler command */
	temp_command=find_command(global_service_event_handler,NULL);

	/* if there is no valid command, exit */
	if(temp_command==NULL)
		return ERROR;

	/* get the raw command line to execute */
	strncpy(raw_command_line,temp_command->command_line,sizeof(raw_command_line));
	raw_command_line[sizeof(raw_command_line)-1]='\x0';

#ifdef DEBUG3
	printf("\tRaw global service event handler command line: %s\n",raw_command_line);
#endif

	/* process any macros in the raw command line */
	process_macros(raw_command_line,processed_command_line,(int)sizeof(processed_command_line));

#ifdef DEBUG3
	printf("\tProcessed global service event handler command line: %s\n",processed_command_line);
#endif

	if(log_event_handlers){
		snprintf(temp_buffer,sizeof(temp_buffer),"GLOBAL SERVICE EVENT HANDLER: %s;%s;%s;%s;%s;%s\n",svc->host_name,svc->description,macro_service_state,macro_state_type,macro_current_service_attempt,global_service_event_handler);
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		write_to_log(temp_buffer);
	        }

	/* run the command */
	my_system(processed_command_line);

#ifdef DEBUG0
	printf("run_global_service_event_handler() end\n");
#endif

	return OK;
        }



/* runs a service event handler command */
int run_service_event_handler(service *svc,command *cmd,int state_type){
	char raw_command_line[MAX_COMMANDLINE_LENGTH];
	char processed_command_line[MAX_COMMANDLINE_LENGTH];
	char temp_buffer[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("run_service_event_handler() start\n");
#endif

	/* get the raw command line to execute */
	strncpy(raw_command_line,cmd->command_line,sizeof(raw_command_line));
	raw_command_line[sizeof(raw_command_line)-1]='\x0';

#ifdef DEBUG3
	printf("\tRaw service event handler command line: %s\n",raw_command_line);
#endif

	/* process any macros in the raw command line */
	process_macros(raw_command_line,processed_command_line,(int)sizeof(processed_command_line));

#ifdef DEBUG3
	printf("\tProcessed service event handler command line: %s\n",processed_command_line);
#endif

	if(log_event_handlers){
		snprintf(temp_buffer,sizeof(temp_buffer),"SERVICE EVENT HANDLER: %s;%s;%s;%s;%s;%s\n",svc->host_name,svc->description,macro_service_state,macro_state_type,macro_current_service_attempt,svc->event_handler);
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		write_to_log(temp_buffer);
	        }

	/* run the command */
	my_system(processed_command_line);

#ifdef DEBUG0
	printf("run_service_event_handler() end\n");
#endif

	return OK;
        }




/******************************************************************/
/****************** HOST EVENT HANDLER FUNCTIONS ******************/
/******************************************************************/


/* handles a change in the status of a host */
int handle_host_event(host *hst,int state,int state_type){
	command *event_command;

#ifdef DEBUG0
	printf("handle_host_event() start\n");
#endif

	/* update host macros */
	grab_host_macros(hst);

	/* grab the host state type macro */
	strncpy(macro_state_type,(state_type==HARD_STATE)?"HARD":"SOFT",sizeof(macro_state_type)-1);
	macro_state_type[sizeof(macro_state_type)-1]='\x0';

	/* make sure the host state macro is correct */
	if(state==HOST_DOWN)
		strcpy(macro_host_state,"DOWN");
	else if(state==HOST_UNREACHABLE)
		strcpy(macro_host_state,"UNREACHABLE");
	else
		strcpy(macro_host_state,"UP");

	/* run the global host event handler */
	run_global_host_event_handler(hst,state,state_type);

	/* find the host event handler command */
	if(!strcmp(hst->event_handler,""))
		event_command=NULL;
	else
		event_command=find_command(hst->event_handler,NULL);

	/* run the event handler command if there is one */
	if(event_command!=NULL)
		run_host_event_handler(hst,event_command,state,state_type);

	/* check for external commands - the event handler may have given us some directives... */
	check_for_external_commands();

#ifdef DEBUG0
	printf("handle_host_event() end\n");
#endif

	return OK;
        }


/* runs the global host event handler */
int run_global_host_event_handler(host *hst,int state,int state_type){
	char raw_command_line[MAX_COMMANDLINE_LENGTH];
	char processed_command_line[MAX_COMMANDLINE_LENGTH];
	char temp_buffer[MAX_INPUT_BUFFER];
	command *temp_command;

#ifdef DEBUG0
	printf("run_global_host_event_handler() start\n");
#endif

	/* find the event handler command */
	temp_command=find_command(global_host_event_handler,NULL);

	/* if there is no valid event handler command, exit */
	if(temp_command==NULL)
		return ERROR;

	/* get the raw command line to execute */
	strcpy(raw_command_line,temp_command->command_line);

#ifdef DEBUG3
	printf("\tRaw global host event handler command line: %s\n",raw_command_line);
#endif

	/* process any macros in the raw command line */
	process_macros(raw_command_line,processed_command_line,(int)sizeof(processed_command_line));

#ifdef DEBUG3
	printf("\tProcessed global host event handler command line: %s\n",processed_command_line);
#endif

	if(log_event_handlers){
		snprintf(temp_buffer,sizeof(temp_buffer),"GLOBAL HOST EVENT HANDLER: %s;%s;%s;%s;%s\n",hst->name,macro_host_state,macro_state_type,macro_current_host_attempt,global_host_event_handler);
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		write_to_log(temp_buffer);
	        }

	/* run the command */
	my_system(processed_command_line);

#ifdef DEBUG0
	printf("run_global_host_event_handler() end\n");
#endif

	return OK;
        }


/* runs a host event handler command */
int run_host_event_handler(host *hst,command *cmd,int state,int state_type){
	char raw_command_line[MAX_COMMANDLINE_LENGTH];
	char processed_command_line[MAX_COMMANDLINE_LENGTH];
	char temp_buffer[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("run_host_event_handler() start\n");
#endif

	/* get the raw command line to execute */
	strcpy(raw_command_line,cmd->command_line);

#ifdef DEBUG3
	printf("\tRaw host event handler command line: %s\n",raw_command_line);
#endif

	/* process any macros in the raw command line */
	process_macros(raw_command_line,processed_command_line,(int)sizeof(processed_command_line));

#ifdef DEBUG3
	printf("\tProcessed host event handler command line: %s\n",processed_command_line);
#endif

	if(log_event_handlers){
		snprintf(temp_buffer,sizeof(temp_buffer),"HOST EVENT HANDLER: %s;%s;%s;%s;%s\n",hst->name,macro_host_state,macro_state_type,macro_current_host_attempt,hst->event_handler);
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		write_to_log(temp_buffer);
	        }

	/* run the command */
	my_system(processed_command_line);

#ifdef DEBUG0
	printf("run_host_event_handler() end\n");
#endif

	return OK;
        }




/******************************************************************/
/**************** SERVICE STATE HANDLER FUNCTIONS *****************/
/******************************************************************/


/* updates service state times */
void update_service_state_times(service *svc){
	unsigned long time_difference;
	time_t current_time;

#ifdef DEBUG0
	printf("update_service_state_times() start\n");
#endif

	/* calculate the time since the last service state change */
	time(&current_time);

	/* if this is NOT the first time we've had a service check/state change.. */
	if(svc->last_state_change!=0L){

		time_difference=(unsigned long)current_time-svc->last_state_change;

		/* use last hard state... */
		if(svc->last_hard_state==STATE_UNKNOWN)
			svc->time_unknown+=time_difference;
		else if(svc->last_hard_state==STATE_WARNING)
			svc->time_warning+=time_difference;
		else if(svc->last_hard_state==STATE_CRITICAL)
			svc->time_critical+=time_difference;
		else
			svc->time_ok+=time_difference;
	        }


	/* update the last service state change time */
	svc->last_state_change=current_time;

	/* update status log with service information */
	update_status_log_with_service(svc);


#ifdef DEBUG0
	printf("update_service_state_times() end\n");
#endif
	
	return;
        }




/******************************************************************/
/****************** HOST STATE HANDLER FUNCTIONS ******************/
/******************************************************************/


/* top level host state handler */
int handle_host_state(host *hst,int state,int state_type){
	int state_change=FALSE;
	char temp_buffer[MAX_INPUT_BUFFER];


#ifdef DEBUG0
	printf("handle_host_state() start\n");
#endif

	/* grab the host macros */
	grab_host_macros(hst);

	/* get the host state type macro */
	strncpy(macro_state_type,(state_type==HARD_STATE)?"HARD":"SOFT",sizeof(macro_state_type));
	macro_state_type[sizeof(macro_state_type)-1]='\x0';

	/* make sure the host state macro is correct */
	if(state==HOST_DOWN)
		strcpy(macro_host_state,"DOWN");
	else if(state==HOST_UNREACHABLE)
		strcpy(macro_host_state,"UNREACHABLE");
	else
		strcpy(macro_host_state,"UP");

	/* has the host state changed? */
	if(hst->status!=state)
		state_change=TRUE;

	/* if the host state has changed... */
	if(state_change==TRUE){

		/* update the host state times */
		if(state_type==HARD_STATE)
			update_host_state_times(hst);

		/* the host just recovered */
		if(state==HOST_UP){

			/* reset the current host attempt number */
			hst->current_attempt=1;
		        }

		/* write the host state change to the main log file */
		if(state_type==HARD_STATE || (state_type==SOFT_STATE && log_host_retries)){
			snprintf(temp_buffer,sizeof(temp_buffer),"HOST ALERT: %s;%s;%s;%s;%s\n",hst->name,(state==HOST_UP)?"RECOVERY":macro_host_state,macro_state_type,macro_current_host_attempt,hst->plugin_output);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			write_to_log(temp_buffer);
		        }

		/* notify contacts about the recovery or problem if its a "hard" state */
		if(state_type==HARD_STATE)
			host_notification(hst,state);

		/* handle the host state change */
		handle_host_event(hst,state,state_type);

		/* set the host state flag and update the status log if this is a hard state */
		if(state_type==HARD_STATE){
			hst->status=state;
			update_status_log_with_host(hst);
		        }
	        }

	/* else the host state has not changed */
	else{

		/* notify contacts if host is still down */
		if(state!=HOST_UP && state_type==HARD_STATE)
			host_notification(hst,state);

		/* if we're in a soft state and we should log host retries, do so now... */
		if(state_type==SOFT_STATE && log_host_retries){

			snprintf(temp_buffer,sizeof(temp_buffer),"HOST ALERT: %s;%s;%s;%s;%s\n",hst->name,(state==HOST_UP)?"RECOVERY":macro_host_state,macro_state_type,macro_current_host_attempt,hst->plugin_output);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			write_to_log(temp_buffer);
		        }

		/* update the status log if this is a hard state */
		if(state_type==HARD_STATE)
			update_status_log_with_host(hst);

	        }

#ifdef DEBUG0
	printf("handle_host_state() end\n");
#endif

	return OK;
        }




/* updates host state times */
void update_host_state_times(host *hst){
	unsigned long time_difference;
	time_t current_time;

#ifdef DEBUG0
	printf("update_host_state_times() start\n");
#endif

	/* get the current time */
	time(&current_time);

	/* if this is NOT the first time we've had a host check/state change... */
	if(hst->last_state_change!=0L){

		time_difference=(unsigned long)current_time-hst->last_state_change;

		if(hst->status==HOST_DOWN)
			hst->time_down+=time_difference;
		else if(hst->status==HOST_UNREACHABLE)
			hst->time_unreachable+=time_difference;
		else
			hst->time_up+=time_difference;
	        }

	/* update the last host state change time */
	hst->last_state_change=current_time;

	/* update status log with host information */
	update_status_log_with_host(hst);


#ifdef DEBUG0
	printf("update_host_state_times() end\n");
#endif
	
	return;
        }

